@@ -287,7 +287,7 @@ class Job(AbstractJob):
         if self.conda_env_file:
             if self._conda_env is None:
                 self._conda_env = self.dag.conda_envs.get(
-                    (self.conda_env_file, self.singularity_img_url)
+                    (self.conda_env_file, self.container_img_url)
                 )
             return self._conda_env
         return None
@@ -304,16 +304,16 @@ class Job(AbstractJob):
 
     @property
     def needs_singularity(self):
-        return self.singularity_img is not None
+        return self.container_img is not None
 
     @property
-    def singularity_img_url(self):
-        return self.rule.singularity_img
+    def container_img_url(self):
+        return self.rule.container_img
 
     @property
-    def singularity_img(self):
-        if self.singularity_img_url:
-            return self.dag.singularity_imgs[self.singularity_img_url]
+    def container_img(self):
+        if self.dag.workflow.use_singularity and self.container_img_url:
+            return self.dag.container_imgs[self.container_img_url]
         return None
 
     @property
@@ -321,8 +321,8 @@ class Job(AbstractJob):
         return self.rule.env_modules
 
     @property
-    def singularity_img_path(self):
-        return self.singularity_img.path if self.singularity_img else None
+    def container_img_path(self):
+        return self.container_img.path if self.container_img else None
 
     @property
     def is_shadow(self):
@@ -410,10 +410,15 @@ class Job(AbstractJob):
             self.is_shell
             or self.is_norun
             or self.is_script
+            or self.is_notebook
             or self.is_wrapper
             or self.is_cwl
         )
 
+    @property
+    def is_pipe(self):
+        return any([is_flagged(o, "pipe") for o in self.output])
+
     @property
     def expanded_output(self):
         """ Iterate over output files while dynamic output is expanded. """
@@ -525,6 +530,10 @@ class Job(AbstractJob):
                 if f in self.dynamic_output:
                     if not self.expand_dynamic(f_):
                         files.add("{} (dynamic)".format(f_))
+                elif is_flagged(f, "pipe"):
+                    # pipe output is always declared as missing
+                    # (even if it might be present on disk for some reason)
+                    files.add(f)
                 elif not f.exists:
                     files.add(f)
 
@@ -1164,9 +1173,8 @@ class GroupJob(AbstractJob):
     def resources(self):
         if self._resources is None:
             self._resources = defaultdict(int)
-            pipe_group = any(
-                [any([is_flagged(o, "pipe") for o in job.output]) for job in self.jobs]
-            )
+            self._resources["_nodes"] = 1
+            pipe_group = any([job.is_pipe for job in self.jobs])
             # iterate over siblings that can be executed in parallel
             for siblings in self.toposorted:
                 sibling_resources = defaultdict(int)
@@ -1395,6 +1403,7 @@ class Reason:
         "noio",
         "nooutput",
         "derived",
+        "pipe",
     ]
 
     def __init__(self):
@@ -1406,6 +1415,7 @@ class Reason:
         self.noio = False
         self.nooutput = False
         self.derived = True
+        self.pipe = False
 
     @lazy_property
     def updated_input(self):
@@ -1470,4 +1480,5 @@ class Reason:
             or self.updated_input_run
             or self.noio
             or self.nooutput
+            or self.pipe
         )
