@@ -48,9 +48,9 @@ class Persistence:
         else:
             self.conda_env_path = os.path.abspath(os.path.expanduser(conda_prefix))
         if singularity_prefix is None:
-            self.singularity_img_path = os.path.join(self.path, "singularity")
+            self.container_img_path = os.path.join(self.path, "singularity")
         else:
-            self.singularity_img_path = os.path.abspath(
+            self.container_img_path = os.path.abspath(
                 os.path.expanduser(singularity_prefix)
             )
         if shadow_prefix is None:
@@ -63,7 +63,7 @@ class Persistence:
             self.shadow_path,
             self.conda_env_archive_path,
             self.conda_env_path,
-            self.singularity_img_path,
+            self.container_img_path,
         ):
             os.makedirs(d, exist_ok=True)
 
@@ -137,7 +137,7 @@ class Persistence:
             shutil.rmtree(self.shadow_path)
             os.mkdir(self.shadow_path)
 
-    def cleanup_conda(self):
+    def conda_cleanup_envs(self):
         # cleanup envs
         in_use = set(env.hash[:8] for env in self.dag.conda_envs.values())
         for d in os.listdir(self.conda_env_path):
@@ -189,7 +189,7 @@ class Persistence:
                     "endtime": endtime,
                     "job_hash": hash(job),
                     "conda_env": conda_env,
-                    "singularity_img_url": job.singularity_img_url,
+                    "container_img_url": job.container_img_url,
                 },
                 f,
             )
