@@ -16,6 +16,7 @@ from operator import attrgetter
 import copy
 import subprocess
 from pathlib import Path
+from urllib.parse import urlparse
 
 from snakemake.logging import logger, format_resources, format_resource_names
 from snakemake.rules import Rule, Ruleorder, RuleProxy
@@ -101,6 +102,8 @@ class Workflow:
         nodes=1,
         cores=1,
         resources=None,
+        conda_cleanup_pkgs=None,
+        edit_notebook=False,
     ):
         """
         Create the controller.
@@ -148,7 +151,7 @@ class Workflow:
         self.singularity_prefix = singularity_prefix
         self.singularity_args = singularity_args
         self.shadow_prefix = shadow_prefix
-        self.global_singularity_img = None
+        self.global_container_img = None
         self.mode = mode
         self.wrapper_prefix = wrapper_prefix
         self.printshellcmds = printshellcmds
@@ -159,8 +162,15 @@ class Workflow:
         self.configfiles = []
         self.run_local = run_local
         self.report_text = None
+        self.conda_cleanup_pkgs = conda_cleanup_pkgs
+        self.edit_notebook = edit_notebook
+        # environment variables to pass to jobs
+        # These are defined via the "envvars:" syntax in the Snakefile itself
+        self.envvars = set()
 
+        self.enable_cache = False
         if cache is not None:
+            self.enable_cache = True
             self.cache_rules = set(cache)
             if self.default_remote_provider is not None:
                 self.output_file_cache = RemoteOutputFileCache(
@@ -191,12 +201,43 @@ class Workflow:
         global checkpoints
         checkpoints = Checkpoints()
 
+    def lint(self, json=False):
+        from snakemake.linting.rules import RuleLinter
+        from snakemake.linting.snakefiles import SnakefileLinter
+
+        json_snakefile_lints, snakefile_linted = SnakefileLinter(
+            self, self.included
+        ).lint(json=json)
+        json_rule_lints, rules_linted = RuleLinter(self, self.rules).lint(json=json)
+
+        linted = snakefile_linted or rules_linted
+
+        if json:
+            import json
+
+            print(
+                json.dumps(
+                    {"snakefiles": json_snakefile_lints, "rules": json_rule_lints},
+                    indent=2,
+                )
+            )
+        else:
+            if not linted:
+                logger.info("Congratulations, your workflow is in a good condition!")
+        return linted
+
     def is_cached_rule(self, rule: Rule):
         return rule.name in self.cache_rules
 
     def get_sources(self):
         files = set()
 
+        def local_path(f):
+            url = urlparse(f)
+            if url.scheme == "file" or url.scheme == "":
+                return url.path
+            return None
+
         def norm_rule_relpath(f, rule):
             if not os.path.isabs(f):
                 f = os.path.join(rule.basedir, f)
@@ -204,7 +245,9 @@ class Workflow:
 
         # get registered sources
         for f in self.included:
-            files.add(os.path.relpath(f))
+            f = local_path(f)
+            if f:
+                files.add(os.path.relpath(f))
         for rule in self.rules:
             script_path = rule.script or rule.notebook
             if script_path:
@@ -217,8 +260,11 @@ class Workflow:
                     for f in files
                 )
             if rule.conda_env:
-                env_path = norm_rule_relpath(rule.conda_env, rule)
-                files.add(env_path)
+                f = local_path(rule.conda_env)
+                if f:
+                    # url points to a local env file
+                    env_path = norm_rule_relpath(f, rule)
+                    files.add(env_path)
 
         for f in self.configfiles:
             files.add(f)
@@ -396,10 +442,10 @@ class Workflow:
         drmaa=None,
         drmaa_log_dir=None,
         kubernetes=None,
-        kubernetes_envvars=None,
         tibanna=None,
         tibanna_sfn=None,
         precommand="",
+        tibanna_config=False,
         container_image=None,
         stats=None,
         force_incomplete=False,
@@ -422,7 +468,7 @@ class Workflow:
         notemp=False,
         nodeps=False,
         cleanup_metadata=None,
-        cleanup_conda=False,
+        conda_cleanup_envs=False,
         cleanup_shadow=False,
         cleanup_scripts=True,
         subsnakemake=None,
@@ -436,10 +482,11 @@ class Workflow:
         greediness=1.0,
         no_hooks=False,
         force_use_threads=False,
-        create_envs_only=False,
+        conda_create_envs_only=False,
         assume_shared_fs=True,
         cluster_status=None,
         report=None,
+        report_stylesheet=None,
         export_cwl=False,
         batch=None,
         keepincomplete=False,
@@ -515,7 +562,7 @@ class Workflow:
             targetrules=targetrules,
             # when cleaning up conda, we should enforce all possible jobs
             # since their envs shall not be deleted
-            forceall=forceall or cleanup_conda,
+            forceall=forceall or conda_cleanup_envs,
             forcefiles=forcefiles,
             forcerules=forcerules,
             priorityfiles=priorityfiles,
@@ -621,7 +668,9 @@ class Workflow:
                         subworkflow.snakefile,
                         workdir=subworkflow.workdir,
                         targets=subworkflow_targets,
-                        configfiles=[subworkflow.configfile],
+                        configfiles=[subworkflow.configfile]
+                        if subworkflow.configfile
+                        else None,
                         updated_files=updated,
                     ):
                         return False
@@ -677,7 +726,7 @@ class Workflow:
         elif report:
             from snakemake.report import auto_report
 
-            auto_report(dag, report)
+            auto_report(dag, report, stylesheet=report_stylesheet)
             return True
         elif printd3dag:
             dag.d3dag()
@@ -734,16 +783,16 @@ class Workflow:
 
         if self.use_singularity:
             if assume_shared_fs:
-                dag.pull_singularity_imgs(
+                dag.pull_container_imgs(
                     dryrun=dryrun or list_conda_envs, quiet=list_conda_envs
                 )
         if self.use_conda:
             if assume_shared_fs:
                 dag.create_conda_envs(
-                    dryrun=dryrun or list_conda_envs or cleanup_conda,
+                    dryrun=dryrun or list_conda_envs or conda_cleanup_envs,
                     quiet=list_conda_envs,
                 )
-            if create_envs_only:
+            if conda_create_envs_only:
                 return True
 
         if list_conda_envs:
@@ -752,14 +801,14 @@ class Workflow:
                 if env:
                     print(
                         simplify_path(env.file),
-                        env.singularity_img_url or "",
+                        env.container_img_url or "",
                         simplify_path(env.path),
                         sep="\t",
                     )
             return True
 
-        if cleanup_conda:
-            self.persistence.cleanup_conda()
+        if conda_cleanup_envs:
+            self.persistence.conda_cleanup_envs()
             return True
 
         scheduler = JobScheduler(
@@ -781,10 +830,10 @@ class Workflow:
             drmaa=drmaa,
             drmaa_log_dir=drmaa_log_dir,
             kubernetes=kubernetes,
-            kubernetes_envvars=kubernetes_envvars,
             tibanna=tibanna,
             tibanna_sfn=tibanna_sfn,
             precommand=precommand,
+            tibanna_config=tibanna_config,
             container_image=container_image,
             printreason=printreason,
             printshellcmds=printshellcmds,
@@ -804,16 +853,21 @@ class Workflow:
                     logger.resources_info(
                         "Provided cluster nodes: {}".format(self.nodes)
                     )
+                elif kubernetes or tibanna:
+                    logger.resources_info("Provided cloud nodes: {}".format(self.nodes))
                 else:
-                    warning = (
-                        "" if self.cores > 1 else " (use --cores to define parallelism)"
-                    )
-                    logger.resources_info(
-                        "Provided cores: {}{}".format(self.cores, warning)
-                    )
-                    logger.resources_info(
-                        "Rules claiming more threads " "will be scaled down."
-                    )
+                    if self.cores is not None:
+                        warning = (
+                            ""
+                            if self.cores > 1
+                            else " (use --cores to define parallelism)"
+                        )
+                        logger.resources_info(
+                            "Provided cores: {}{}".format(self.cores, warning)
+                        )
+                        logger.resources_info(
+                            "Rules claiming more threads " "will be scaled down."
+                        )
 
                 provided_resources = format_resources(self.global_resources)
                 if provided_resources:
@@ -826,7 +880,7 @@ class Workflow:
                     logger.info("Conda environments: ignored")
 
                 if not self.use_singularity and any(
-                    rule.singularity_img for rule in self.rules
+                    rule.container_img for rule in self.rules
                 ):
                     logger.info("Singularity containers: ignored")
 
@@ -877,6 +931,20 @@ class Workflow:
         assert self.included_stack
         return os.path.abspath(os.path.dirname(self.included_stack[-1]))
 
+    def register_envvars(self, *envvars):
+        """
+        Register environment variables that shall be passed to jobs.
+        If used multiple times, union is taken.
+        """
+        undefined = [var for var in envvars if var not in os.environ]
+        if undefined:
+            raise WorkflowError(
+                "The following environment variables are requested by the workflow but undefined. "
+                "Please make sure that they are correctly defined before running Snakemake:\n"
+                "{}".format("\n".join(undefined))
+            )
+        self.envvars.update(envvars)
+
     def include(
         self,
         snakefile,
@@ -1019,7 +1087,9 @@ class Workflow:
                 if name in self.overwrite_threads:
                     rule.resources["_cores"] = self.overwrite_threads[name]
                 else:
-                    rule.resources["_cores"] = int(ruleinfo.threads)
+                    if isinstance(ruleinfo.threads, float):
+                        ruleinfo.threads = int(ruleinfo.threads)
+                    rule.resources["_cores"] = ruleinfo.threads
             if ruleinfo.shadow_depth:
                 if ruleinfo.shadow_depth not in (True, "shallow", "full", "minimal"):
                     raise RuleException(
@@ -1041,10 +1111,16 @@ class Workflow:
                 if args:
                     raise RuleException("Resources have to be named.")
                 if not all(
-                    map(lambda r: isinstance(r, int) or callable(r), resources.values())
+                    map(
+                        lambda r: isinstance(r, int)
+                        or isinstance(r, str)
+                        or callable(r),
+                        resources.values(),
+                    )
                 ):
                     raise RuleException(
-                        "Resources values have to be integers or callables", rule=rule
+                        "Resources values have to be integers, strings, or callables (functions)",
+                        rule=rule,
                     )
                 rule.resources.update(resources)
             if ruleinfo.priority:
@@ -1066,13 +1142,12 @@ class Workflow:
             if not self.run_local and ruleinfo.group is not None:
                 rule.group = ruleinfo.group
             if ruleinfo.wrapper:
-                if self.use_conda:
-                    rule.conda_env = snakemake.wrapper.get_conda_env(
-                        ruleinfo.wrapper, prefix=self.wrapper_prefix
-                    )
+                rule.conda_env = snakemake.wrapper.get_conda_env(
+                    ruleinfo.wrapper, prefix=self.wrapper_prefix
+                )
                 # TODO retrieve suitable singularity image
 
-            if self.use_env_modules and ruleinfo.env_modules:
+            if ruleinfo.env_modules:
                 # If using environment modules and they are defined for the rule,
                 # ignore conda and singularity directive below.
                 # The reason is that this is likely intended in order to use
@@ -1086,56 +1161,55 @@ class Workflow:
                 )
                 if invalid_rule:
                     raise RuleException(
-                        "Modules directive is only allowed with "
+                        "envmodules directive is only allowed with "
                         "shell, script, notebook, or wrapper directives (not with run)",
                         rule=rule,
                     )
                 from snakemake.deployment.env_modules import EnvModules
 
                 rule.env_modules = EnvModules(*ruleinfo.env_modules)
-            else:
-                if ruleinfo.conda_env and self.use_conda:
-                    if not (
-                        ruleinfo.script
-                        or ruleinfo.wrapper
-                        or ruleinfo.shellcmd
-                        or ruleinfo.notebook
-                    ):
-                        raise RuleException(
-                            "Conda environments are only allowed "
-                            "with shell, script, notebook, or wrapper directives "
-                            "(not with run).",
-                            rule=rule,
-                        )
-                    if not (
-                        urllib.parse.urlparse(ruleinfo.conda_env).scheme
-                        or os.path.isabs(ruleinfo.conda_env)
-                    ):
-                        ruleinfo.conda_env = os.path.join(
-                            self.current_basedir, ruleinfo.conda_env
-                        )
-                    rule.conda_env = ruleinfo.conda_env
 
-                if self.use_singularity:
-                    invalid_rule = not (
-                        ruleinfo.script
-                        or ruleinfo.wrapper
-                        or ruleinfo.shellcmd
-                        or ruleinfo.notebook
+            if ruleinfo.conda_env:
+                if not (
+                    ruleinfo.script
+                    or ruleinfo.wrapper
+                    or ruleinfo.shellcmd
+                    or ruleinfo.notebook
+                ):
+                    raise RuleException(
+                        "Conda environments are only allowed "
+                        "with shell, script, notebook, or wrapper directives "
+                        "(not with run).",
+                        rule=rule,
                     )
-                    if ruleinfo.singularity_img:
-                        if invalid_rule:
-                            raise RuleException(
-                                "Singularity directive is only allowed "
-                                "with shell, script, notebook or wrapper directives "
-                                "(not with run).",
-                                rule=rule,
-                            )
-                        rule.singularity_img = ruleinfo.singularity_img
-                    elif self.global_singularity_img:
-                        if not invalid_rule:
-                            # skip rules with run directive
-                            rule.singularity_img = self.global_singularity_img
+                if not (
+                    urllib.parse.urlparse(ruleinfo.conda_env).scheme
+                    or os.path.isabs(ruleinfo.conda_env)
+                ):
+                    ruleinfo.conda_env = os.path.join(
+                        self.current_basedir, ruleinfo.conda_env
+                    )
+                rule.conda_env = ruleinfo.conda_env
+
+            invalid_rule = not (
+                ruleinfo.script
+                or ruleinfo.wrapper
+                or ruleinfo.shellcmd
+                or ruleinfo.notebook
+            )
+            if ruleinfo.container_img:
+                if invalid_rule:
+                    raise RuleException(
+                        "Singularity directive is only allowed "
+                        "with shell, script, notebook or wrapper directives "
+                        "(not with run).",
+                        rule=rule,
+                    )
+                rule.container_img = ruleinfo.container_img
+            elif self.global_container_img:
+                if not invalid_rule:
+                    # skip rules with run directive
+                    rule.container_img = self.global_container_img
 
             rule.norun = ruleinfo.norun
             rule.docstring = ruleinfo.docstring
@@ -1148,6 +1222,21 @@ class Workflow:
             rule.restart_times = self.restart_times
             rule.basedir = self.current_basedir
 
+            if ruleinfo.cache is True:
+                if not self.enable_cache:
+                    logger.warning(
+                        "Workflow defines that rule {} is eligible for caching between workflows "
+                        "(use the --cache argument to enable this).".format(rule.name)
+                    )
+                else:
+                    self.cache_rules.add(rule.name)
+            elif not (ruleinfo.cache is False):
+                raise WorkflowError(
+                    "Invalid argument for 'cache:' directive. Only true allowed. "
+                    "To deactivate caching, remove directive.",
+                    rule=rule,
+                )
+
             ruleinfo.func.__name__ = "__{}".format(rule.name)
             self.globals[ruleinfo.func.__name__] = ruleinfo.func
             setattr(rules, rule.name, RuleProxy(rule))
@@ -1195,6 +1284,13 @@ class Workflow:
 
         return decorate
 
+    def cache_rule(self, cache):
+        def decorate(ruleinfo):
+            ruleinfo.cache = cache
+            return ruleinfo
+
+        return decorate
+
     def message(self, message):
         def decorate(ruleinfo):
             ruleinfo.message = message
@@ -1216,9 +1312,9 @@ class Workflow:
 
         return decorate
 
-    def singularity(self, singularity_img):
+    def container(self, container_img):
         def decorate(ruleinfo):
-            ruleinfo.singularity_img = singularity_img
+            ruleinfo.container_img = container_img
             return ruleinfo
 
         return decorate
@@ -1230,8 +1326,8 @@ class Workflow:
 
         return decorate
 
-    def global_singularity(self, singularity_img):
-        self.global_singularity_img = singularity_img
+    def global_container(self, container_img):
+        self.global_container_img = container_img
 
     def threads(self, threads):
         def decorate(ruleinfo):
@@ -1343,7 +1439,7 @@ class RuleInfo:
         self.message = None
         self.benchmark = None
         self.conda_env = None
-        self.singularity_img = None
+        self.container_img = None
         self.env_modules = None
         self.wildcard_constraints = None
         self.threads = None
@@ -1358,6 +1454,7 @@ class RuleInfo:
         self.notebook = None
         self.wrapper = None
         self.cwl = None
+        self.cache = False
 
 
 class Subworkflow:
