@@ -3,173 +3,12 @@ __copyright__ = "Copyright 2015-2019, Johannes Köster"
 __email__ = "koester@jimmy.harvard.edu"
 __license__ = "MIT"
 
-import sys
 import os
-import shutil
-from os.path import join
-from subprocess import call
-import tempfile
-import hashlib
-import urllib
-from shutil import rmtree, which
-from shlex import quote
-import pytest
-import subprocess
+import sys
 
-from snakemake import snakemake
-from snakemake.shell import shell
+sys.path.insert(0, os.path.dirname(__file__))
 
-
-def dpath(path):
-    """get path to a data file (relative to the directory this
-	test lives in)"""
-    return os.path.realpath(join(os.path.dirname(__file__), path))
-
-
-def md5sum(filename):
-    data = open(filename, "rb").read()
-    return hashlib.md5(data).hexdigest()
-
-
-# test skipping
-def is_connected():
-    try:
-        urllib.request.urlopen("http://www.google.com", timeout=1)
-        return True
-    except urllib.request.URLError:
-        return False
-
-
-def is_ci():
-    return "CI" in os.environ
-
-
-def has_gcloud_service_key():
-    return "GCLOUD_SERVICE_KEY" in os.environ
-
-
-def has_gcloud_cluster():
-    return "GCLOUD_CLUSTER" in os.environ
-
-
-gcloud = pytest.mark.skipif(
-    not is_connected() or not has_gcloud_service_key() or not has_gcloud_cluster(),
-    reason="Skipping GCLOUD tests because not on "
-    "CI, no inet connection or not logged "
-    "in to gcloud.",
-)
-
-connected = pytest.mark.skipif(not is_connected(), reason="no internet connection")
-
-ci = pytest.mark.skipif(not is_ci(), reason="not in CI")
-not_ci = pytest.mark.skipif(is_ci(), reason="skipped in CI")
-
-
-def copy(src, dst):
-    if os.path.isdir(src):
-        shutil.copytree(src, os.path.join(dst, os.path.basename(src)))
-    else:
-        shutil.copy(src, dst)
-
-
-def run(
-    path,
-    shouldfail=False,
-    snakefile="Snakefile",
-    subpath=None,
-    no_tmpdir=False,
-    check_md5=True,
-    cores=3,
-    set_pythonpath=True,
-    cleanup=True,
-    **params
-):
-    """
-    Test the Snakefile in path.
-    There must be a Snakefile in the path and a subdirectory named
-    expected-results. If cleanup is False, we return the temporary
-    directory to the calling test for inspection, and the test should
-    clean it up.
-    """
-    if set_pythonpath:
-        # Enforce current workdir (the snakemake source dir) to also be in PYTHONPATH
-        # when subprocesses are invoked in the tempdir defined below.
-        os.environ["PYTHONPATH"] = os.getcwd()
-    elif "PYTHONPATH" in os.environ:
-        del os.environ["PYTHONPATH"]
-
-    results_dir = join(path, "expected-results")
-    snakefile = join(path, snakefile)
-    assert os.path.exists(snakefile)
-    assert os.path.exists(results_dir) and os.path.isdir(
-        results_dir
-    ), "{} does not exist".format(results_dir)
-
-    # If we need to further check results, we won't cleanup tmpdir
-    tmpdir = next(tempfile._get_candidate_names())
-    tmpdir = os.path.join(tempfile.gettempdir(), "snakemake-%s" % tmpdir)
-    os.mkdir(tmpdir)
-
-    config = {}
-
-    # handle subworkflow
-    if subpath is not None:
-        # set up a working directory for the subworkflow and pass it in `config`
-        # for now, only one subworkflow is supported
-        assert os.path.exists(subpath) and os.path.isdir(
-            subpath
-        ), "{} does not exist".format(subpath)
-        subworkdir = os.path.join(tmpdir, "subworkdir")
-        os.mkdir(subworkdir)
-
-        # copy files
-        for f in os.listdir(subpath):
-            copy(os.path.join(subpath, f), subworkdir)
-        config["subworkdir"] = subworkdir
-
-    # copy files
-    for f in os.listdir(path):
-        print(f)
-        copy(os.path.join(path, f), tmpdir)
-
-    # run snakemake
-    success = snakemake(
-        snakefile,
-        cores=cores,
-        workdir=path if no_tmpdir else tmpdir,
-        stats="stats.txt",
-        config=config,
-        verbose=True,
-        **params
-    )
-    if shouldfail:
-        assert not success, "expected error on execution"
-    else:
-        assert success, "expected successful execution"
-        for resultfile in os.listdir(results_dir):
-            if resultfile in [".gitignore", ".gitkeep"] or not os.path.isfile(
-                os.path.join(results_dir, resultfile)
-            ):
-                # this means tests cannot use directories as output files
-                continue
-            targetfile = join(tmpdir, resultfile)
-            expectedfile = join(results_dir, resultfile)
-            assert os.path.exists(targetfile), 'expected file "{}" not produced'.format(
-                resultfile
-            )
-            if check_md5:
-                # if md5sum(targetfile) != md5sum(expectedfile):
-                #     import pdb; pdb.set_trace()
-                if md5sum(targetfile) != md5sum(expectedfile):
-                    with open(targetfile) as target:
-                        content = target.read()
-                    assert False, 'wrong result produced for file "{}":\n{}'.format(
-                        resultfile, content
-                    )
-
-    if not cleanup:
-        return tmpdir
-    shutil.rmtree(tmpdir)
+from common import *
 
 
 def test_list_untracked():
@@ -250,6 +89,7 @@ def test13():
 
 
 def test14():
+    os.environ["TESTVAR"] = "test"
     run(dpath("test14"), snakefile="Snakefile.nonstandard", cluster="./qsub")
 
 
@@ -278,7 +118,16 @@ def test_ancient():
 
 
 def test_report():
-    run(dpath("test_report"), report="report.html", check_md5=False)
+    run(
+        dpath("test_report"),
+        report="report.html",
+        report_stylesheet="custom-stylesheet.css",
+        check_md5=False,
+    )
+
+
+def test_report_zip():
+    run(dpath("test_report_zip"), report="report.zip", check_md5=False)
 
 
 def test_dynamic():
@@ -395,6 +244,7 @@ def test_remote():
 
 
 def test_cluster_sync():
+    os.environ["TESTVAR"] = "test"
     run(dpath("test14"), snakefile="Snakefile.nonstandard", cluster_sync="./qsub")
 
 
@@ -1038,21 +888,6 @@ def test_pipes2():
     run(dpath("test_pipes2"))
 
 
-@pytest.mark.skip(
-    reason="The AWS Access Key Id you provided does not exist in our records."
-)
-def test_tibanna():
-    workdir = dpath("test_tibanna")
-    subprocess.check_call(["python", "cleanup.py"], cwd=workdir)
-    run(
-        workdir,
-        use_conda=True,
-        configfiles=[os.path.join(workdir, "config.json")],
-        default_remote_prefix="snakemake-tibanna-test/1",
-        tibanna_sfn="tibanna_unicorn_johannes",
-    )
-
-
 def test_expand_flag():
     run(dpath("test_expand_flag"), shouldfail=True)
 
@@ -1120,6 +955,12 @@ def test_github_issue78():
     run(dpath("test_github_issue78"), use_singularity=True)
 
 
+def test_envvars():
+    run(dpath("test_envvars"), shouldfail=True)
+    os.environ["TEST_ENV_VAR"] = "test"
+    run(dpath("test_envvars"))
+
+
 def test_github_issue105():
     run(dpath("test_github_issue105"))
 
@@ -1127,7 +968,7 @@ def test_github_issue105():
 def test_output_file_cache():
     test_path = dpath("test_output_file_cache")
     os.environ["SNAKEMAKE_OUTPUT_CACHE"] = os.path.join(test_path, "cache")
-    run(test_path, cache=["a", "b", "c"])
+    run(test_path, cache=["a", "b"])
     run(test_path, cache=["invalid_multi"], targets="invalid1.txt", shouldfail=True)
 
 
@@ -1152,3 +993,28 @@ def test_core_dependent_threads():
 
 def test_env_modules():
     run(dpath("test_env_modules"), use_env_modules=True)
+
+
+@connected
+def test_container():
+    run(dpath("test_container"), use_singularity=True)
+
+
+def test_linting():
+    snakemake(
+        snakefile=os.path.join(dpath("test14"), "Snakefile.nonstandard"), lint=True
+    )
+
+
+def test_string_resources():
+    from snakemake.resources import DefaultResources
+
+    run(
+        dpath("test_string_resources"),
+        default_resources=DefaultResources(["gpu_model='nvidia-tesla-1000'"]),
+        cluster="./qsub.py",
+    )
+
+
+def test_jupyter_notebook():
+    run(dpath("test_jupyter_notebook"), use_conda=True)
